"{ Package: 'stx:goodies/xmlsuite/xquery' }"

"{ NameSpace: XQuery }"

TestCase subclass:#XQueryDocumentAdaptorTests
	instanceVariableNames:'xqueryExecutor documentAdaptor'
	classVariableNames:''
	poolDictionaries:''
	category:'XQuery-Tests'
!


!XQueryDocumentAdaptorTests class methodsFor:'accessing'!

resources

    ^Array 
        with:XQueryDocumentProviderTestResource

    "Created: / 06-04-2007 / 17:19:19 / janfrog"
! !

!XQueryDocumentAdaptorTests class methodsFor:'defaults'!

testedURI
    ^'file:///bookstore.xml'

    "Created: / 25-10-2006 / 09:12:04 / ked"
    "Modified: / 02-12-2006 / 12:08:35 / ked"
    "Modified: / 06-04-2007 / 17:21:40 / janfrog"
! !

!XQueryDocumentAdaptorTests methodsFor:'initialization & release'!

setUp

    xqueryExecutor := XQueryExecutor new
                        setDocumentProvider: XQueryDocumentProviderTestResource current;
                        yourself.

    documentAdaptor := (xqueryExecutor documentAdaptorsFor: self class testedURI) first.

    "Created: / 25-10-2006 / 09:09:21 / ked"
    "Modified: / 20-09-2007 / 10:34:33 / janfrog"
!

tearDown

    xqueryExecutor releaseDocumentAdaptors.

    xqueryExecutor := nil.

    "Created: / 25-10-2006 / 09:09:21 / ked"
! !

!XQueryDocumentAdaptorTests methodsFor:'tests'!

testAncestor
    "Selects all ancestors (parent, grandparent, etc.) of the current node"

    | childNode ancestorsCollection |

    childNode := (XQueryAccessedNode new documentAdaptor:documentAdaptor; xpathDocument) first  xpathChild first  xpathChild first  xpathChild first .
    ancestorsCollection := childNode xpathAncestor.

    self assert: ancestorsCollection size = 3.

    "reverse document order"
    self assert:(ancestorsCollection third  xpathIsDocument).
    self assert:(ancestorsCollection second  xpathName) = 'bookstore'.
    self assert:(ancestorsCollection second  xpathIsElement).
    self assert:(ancestorsCollection first  xpathName) = 'book'.
    self assert:(ancestorsCollection first  xpathIsElement).

    childNode := (XQueryAccessedNode new documentAdaptor:documentAdaptor; xpathDocument) first.
    ancestorsCollection := childNode xpathAncestor.

    self assert: ancestorsCollection size = 0.

    "Created: / 25-10-2006 / 09:06:43 / ked"
    "Modified: / 02-11-2006 / 12:56:36 / ked"
    "Modified: / 05-12-2007 / 21:44:03 / janfrog"
!

testAncestorOrSelf
    "Selects all ancestors (parent, grandparent, etc.) of the current node and the current node itself"
    
    |childNode ancestorsCollection|

    childNode := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild first 
                xpathChild first.
    ancestorsCollection := childNode xpathAncestorOrSelf.
    self assert:ancestorsCollection size = 4.
     "reverse document order"
    self assert:(ancestorsCollection fourth xpathIsDocument).
    self assert:(ancestorsCollection third xpathName) = 'bookstore'.
    self assert:(ancestorsCollection third xpathIsElement).
    self assert:(ancestorsCollection second xpathName) = 'book'.
    self assert:(ancestorsCollection second xpathIsElement).
    self assert:(ancestorsCollection first xpathName) = 'title'.
    self assert:(ancestorsCollection first xpathIsElement).
    childNode := childNode xpathDocument first.
    ancestorsCollection := childNode xpathAncestorOrSelf.
    self assert:ancestorsCollection size = 1.
     "reverse document order"
    self assert:(ancestorsCollection first xpathIsDocument).

    "Created: / 25-10-2006 / 09:06:43 / ked"
    "Modified: / 02-11-2006 / 12:57:00 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testAttribute
    "Selects all attributes of the current node"
    
    |childNode attributesCollection attributeNode|

    childNode := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild first 
                xpathChild first.
    attributesCollection := childNode xpathAttribute.
    self assert:attributesCollection size = 2.
    self assert:(attributesCollection 
                anySatisfy:[:node | node xpathName = 'lang' and:[ node xpathValue = 'eng' ] ]).
    self assert:(attributesCollection 
                anySatisfy:[:node | node xpathName = 'withPictures' and:[ node xpathValue = 'yes' ] ]).
    attributeNode := (attributesCollection 
                select:[:node | node xpathName = 'lang' ]) first.
     " TODO - nikoli podle XDM
     self should: [ attributeNodeHandlingCollection := dataAccessor xpathAncestorOf: attributeNode. ] raise: Exception.
    "" TODO - mozna ano, mozna ne
     self should: [ attributeNodeHandlingCollection := attributeNode xpathFollowingSibling. ] raise: Exception.
    "
    childNode := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild third 
                xpathChild first.
    attributesCollection := childNode xpathAttribute.
    self assert:attributesCollection size = 0.

    "Created: / 25-10-2006 / 09:06:43 / ked"
    "Modified: / 02-11-2006 / 12:55:40 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testChild
    "Selects all children of the current node"
    
    |parentNode childrenCollection anotherChildrenCollection|

    parentNode := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild first.
    childrenCollection := parentNode xpathChild.
    self assert:childrenCollection size = 4.
     "document order"
    self assert:(childrenCollection first xpathName) = 'title'.
    self assert:(childrenCollection first xpathIsElement).
    self assert:(childrenCollection second xpathName) = 'price'.
    self assert:(childrenCollection second xpathIsElement).
    self assert:(childrenCollection third xpathName) = 'parentalAdvisory'.
    self assert:(childrenCollection third xpathIsElement).
    self assert:(childrenCollection fourth xpathName) = 'description'.
    self assert:(childrenCollection fourth xpathIsElement).
    anotherChildrenCollection := childrenCollection first xpathChild.
    self assert:anotherChildrenCollection size = 1.
    self assert:(anotherChildrenCollection first xpathValue) = 'Harry Potter'.
    self assert:(anotherChildrenCollection first xpathIsText).
    anotherChildrenCollection := anotherChildrenCollection first xpathChild.
    self assert:anotherChildrenCollection size = 0.
    anotherChildrenCollection := childrenCollection fourth xpathChild.
    self assert:anotherChildrenCollection size = 0.

    "Created: / 25-10-2006 / 09:06:43 / ked"
    "Modified: / 02-11-2006 / 12:57:42 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testDescendant
    "Selects all descendants (children, grandchildren, etc.) of the current node"
    
    |parentNode descendantsCollection anotherDescendantsCollection|

    parentNode := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild first.
    descendantsCollection := parentNode xpathDescendant.
    self assert:descendantsCollection size = 7.
     "document order"
    self assert:(descendantsCollection first xpathName) = 'title'.
    self assert:(descendantsCollection first xpathIsElement).
    self assert:(descendantsCollection second xpathValue) = 'Harry Potter'.
    self assert:(descendantsCollection second xpathIsText).
    self assert:(descendantsCollection third xpathName) = 'price'.
    self assert:(descendantsCollection third xpathIsElement).
    self assert:(descendantsCollection fourth xpathValue) = '29.99'.
    self assert:(descendantsCollection fourth xpathIsText).
    self assert:(descendantsCollection fifth xpathName) = 'parentalAdvisory'.
    self assert:(descendantsCollection fifth xpathIsElement).
    self assert:(descendantsCollection sixth xpathValue) = '50+'.
    self assert:(descendantsCollection sixth xpathIsText).
    self assert:(descendantsCollection seventh xpathName) = 'description'.
    self assert:(descendantsCollection seventh xpathIsElement).
    anotherDescendantsCollection := descendantsCollection fifth 
                xpathDescendant.
    self assert:anotherDescendantsCollection size = 1.
    anotherDescendantsCollection := descendantsCollection sixth 
                xpathDescendant.
    self assert:anotherDescendantsCollection size = 0.
    anotherDescendantsCollection := descendantsCollection seventh 
                xpathDescendant.
    self assert:anotherDescendantsCollection size = 0.

    "Created: / 25-10-2006 / 09:06:43 / ked"
    "Modified: / 02-11-2006 / 12:58:07 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testDescendantOrSelf
    "Selects all descendants (children, grandchildren, etc.) of the current node and the current node itself"
    
    |parentNode descendantsCollection anotherDescendantsCollection|

    parentNode := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild second.
    descendantsCollection := parentNode xpathDescendantOrSelf.
    self assert:descendantsCollection size = 5.
     "document order"
    self assert:(descendantsCollection first xpathName) = 'book'.
    self assert:(descendantsCollection first xpathIsElement).
    self assert:(descendantsCollection second xpathName) = 'title'.
    self assert:(descendantsCollection second xpathIsElement).
    self assert:(descendantsCollection third xpathValue) = 'Learning XML'.
    self assert:(descendantsCollection third xpathIsText).
    self assert:(descendantsCollection fourth xpathName) = 'price'.
    self assert:(descendantsCollection fourth xpathIsElement).
    self assert:(descendantsCollection fifth xpathValue) = '39.95'.
    self assert:(descendantsCollection fifth xpathIsText).
    anotherDescendantsCollection := descendantsCollection fourth 
                xpathDescendantOrSelf.
    self assert:anotherDescendantsCollection size = 2.
    anotherDescendantsCollection := descendantsCollection fifth 
                xpathDescendantOrSelf.
    self assert:anotherDescendantsCollection size = 1.

    "Created: / 25-10-2006 / 09:06:43 / ked"
    "Modified: / 02-11-2006 / 12:58:24 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testDocument
    "Retrieves document element of the document"
    
    |documentCollection|

    documentCollection := (XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument.
    self assert:documentCollection size = 1.
    self assert:(documentCollection first xpathIsDocument).

    "Created: / 25-10-2006 / 09:06:44 / ked"
    "Modified: / 28-10-2006 / 23:37:11 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testFollowing
    "Selects everything in the document after the closing tag of the current node"
    
    |node followingsCollection|

    node := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild second 
                xpathChild second.
    self assert:(node xpathName) = 'price'.
    self assert:(node xpathIsElement).
    followingsCollection := node xpathFollowing.
    self assert:followingsCollection size = 5.
     "document order"
    self assert:(followingsCollection first xpathName) = 'book'.
    self assert:(followingsCollection first xpathIsElement).
    self assert:(followingsCollection second xpathName) = 'title'.
    self assert:(followingsCollection second xpathIsElement).
    self assert:(followingsCollection third xpathValue) = '1984'.
    self assert:(followingsCollection third xpathIsText).
    self assert:(followingsCollection fourth xpathName) = 'price'.
    self assert:(followingsCollection fourth xpathIsElement).
    self assert:(followingsCollection fifth xpathValue) = '19.84'.
    self assert:(followingsCollection fifth xpathIsText).
    node := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild third 
                xpathChild second.
    self assert:(node xpathName) = 'price'.
    self assert:(node xpathIsElement).
    followingsCollection := node xpathFollowing.
    self assert:followingsCollection size = 0.

    "Created: / 25-10-2006 / 09:06:44 / ked"
    "Modified: / 02-11-2006 / 12:58:59 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testFollowingSibling
    "Selects all siblings after the current node"
    
    |node followingSiblingsCollection anotherFollowingSiblingsCollection|

    node := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild first 
                xpathChild first.
    self assert:(node xpathName) = 'title'.
    self assert:(node xpathIsElement).
    followingSiblingsCollection := node xpathFollowingSibling.
    self assert:followingSiblingsCollection size = 3.
     "document order"
    self assert:(followingSiblingsCollection first xpathName) = 'price'.
    self assert:(followingSiblingsCollection first xpathIsElement).
    self 
        assert:(followingSiblingsCollection second xpathName) = 'parentalAdvisory'.
    self assert:(followingSiblingsCollection second xpathIsElement).
    self assert:(followingSiblingsCollection third xpathName) = 'description'.
    self assert:(followingSiblingsCollection third xpathIsElement).
    anotherFollowingSiblingsCollection := followingSiblingsCollection third 
                xpathFollowingSibling.
    self assert:anotherFollowingSiblingsCollection size = 0.

    "Created: / 25-10-2006 / 09:06:44 / ked"
    "Modified: / 02-11-2006 / 13:29:37 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testParent
    "Selects the parent of the current node"
    
    |childrenCollection parentNode documentNode|

    childrenCollection := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild.
    parentNode := childrenCollection first xpathParent first.
    self assert:(parentNode xpathIsElement).
    self assert:(parentNode xpathName) = 'bookstore'.
     "parentNode := dataAccessor to: childrenCollection apply:#(first getChildPointersOf: first getChildPointersOf: first getParentPointerOf:)."
    parentNode := childrenCollection first xpathChild first xpathChild first 
                xpathParent first.
    self assert:(parentNode xpathIsElement).
    self assert:(parentNode xpathName) = 'title'.
    documentNode := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first.
    parentNode := documentNode xpathParent.
    self assert:parentNode size = 0.

    "Created: / 25-10-2006 / 09:06:44 / ked"
    "Modified: / 02-11-2006 / 13:29:52 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testPreceding
    "Selects everything in the document that is before the start tag of the current node"
    
    |node precedingsCollection|

    node := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild second 
                xpathChild second.
    self assert:(node xpathName) = 'price'.
    self assert:(node xpathIsElement).
    precedingsCollection := node xpathPreceding.
    self assert:precedingsCollection size = 10.
     "document order"
    self assert:(precedingsCollection first xpathName) = 'title'.
    self assert:(precedingsCollection first xpathIsElement).
    self assert:(precedingsCollection second xpathValue) = 'Learning XML'.
    self assert:(precedingsCollection second xpathIsText).
    self assert:(precedingsCollection third xpathName) = 'book'.
    self assert:(precedingsCollection third xpathIsElement).
    self assert:(precedingsCollection fourth xpathName) = 'title'.
    self assert:(precedingsCollection fourth xpathIsElement).
    self assert:(precedingsCollection fifth xpathValue) = 'Harry Potter'.
    self assert:(precedingsCollection fifth xpathIsText).
    self assert:(precedingsCollection sixth xpathName) = 'price'.
    self assert:(precedingsCollection sixth xpathIsElement).
    self assert:(precedingsCollection seventh xpathValue) = '29.99'.
    self assert:(precedingsCollection seventh xpathIsText).
    self assert:((precedingsCollection at:8) xpathName) = 'parentalAdvisory'.
    self assert:((precedingsCollection at:8) xpathIsElement).
    self assert:((precedingsCollection at:9) xpathValue) = '50+'.
    self assert:((precedingsCollection at:9) xpathIsText).
    self assert:((precedingsCollection at:10) xpathName) = 'description'.
    self assert:((precedingsCollection at:10) xpathIsElement).
    node := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first.
    precedingsCollection := node xpathPreceding.
    self assert:precedingsCollection size = 0.
    node := node xpathChild first.
    precedingsCollection := node xpathPreceding.
    self assert:precedingsCollection size = 0.

    "Created: / 25-10-2006 / 09:06:44 / ked"
    "Modified: / 02-11-2006 / 13:52:09 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
!

testPrecedingSibling
    "Selects all siblings before the current node"
    
    |node precedingSiblingsCollection|

    node := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first 
                xpathChild first 
                xpathChild second 
                xpathChild second.
    self assert:(node xpathName) = 'price'.
    self assert:(node xpathIsElement).
    precedingSiblingsCollection := node xpathPrecedingSibling.
    self assert:precedingSiblingsCollection size = 1.
    self assert:(precedingSiblingsCollection first xpathName) = 'title'.
    self assert:(precedingSiblingsCollection first xpathIsElement).
    node := ((XQueryAccessedNode new)
                documentAdaptor:documentAdaptor;
                xpathDocument) first.
    precedingSiblingsCollection := node xpathPrecedingSibling.
    self assert:precedingSiblingsCollection size = 0.
    node := node xpathChild first.
    precedingSiblingsCollection := node xpathPrecedingSibling.
    self assert:precedingSiblingsCollection size = 0.

    "Created: / 25-10-2006 / 09:06:44 / ked"
    "Modified: / 02-11-2006 / 13:52:32 / ked"
    "Modified: / 05-12-2007 / 21:48:54 / janfrog"
! !

!XQueryDocumentAdaptorTests class methodsFor:'documentation'!

version
    ^ '$Header: /opt/data/cvs/stx/goodies/xmlsuite/xquery/XQuery__XQueryDocumentAdaptorTests.st,v 1.5 2007-12-05 21:32:02 vranyj1 Exp $'
! !
